package x86_test

import (
	"testing"

	"git.akyoto.dev/cli/q/src/cpu"
	"git.akyoto.dev/cli/q/src/x86"
	"git.akyoto.dev/go/assert"
)

func TestPushRegister(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Code     []byte
	}{
		{x86.RAX, []byte{0x50}},
		{x86.RCX, []byte{0x51}},
		{x86.RDX, []byte{0x52}},
		{x86.RBX, []byte{0x53}},
		{x86.RSP, []byte{0x54}},
		{x86.RBP, []byte{0x55}},
		{x86.RSI, []byte{0x56}},
		{x86.RDI, []byte{0x57}},
		{x86.R8, []byte{0x41, 0x50}},
		{x86.R9, []byte{0x41, 0x51}},
		{x86.R10, []byte{0x41, 0x52}},
		{x86.R11, []byte{0x41, 0x53}},
		{x86.R12, []byte{0x41, 0x54}},
		{x86.R13, []byte{0x41, 0x55}},
		{x86.R14, []byte{0x41, 0x56}},
		{x86.R15, []byte{0x41, 0x57}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("push %s", pattern.Register)
		code := x86.PushRegister(nil, pattern.Register)
		assert.DeepEqual(t, code, pattern.Code)
	}
}
